<?php
//-------------------------------------------------------------------------
// OVIDENTIA http://www.ovidentia.org
// Ovidentia is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.
// 
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
// USA.
//-------------------------------------------------------------------------
/**
 * @license http://opensource.org/licenses/gpl-license.php GNU General Public License (GPL)
 * @copyright Copyright (c) 2006 by CANTICO ({@link http://www.cantico.fr})
 */
require_once 'base.php';


class NL_DataSourceBase
{
	private $aDatas = array();
	
	function NL_DataSourceBase()
	{
	}
	
	function getNextItem()
	{
		$aDatas = each($this->aDatas);
		if(false !== $aDatas)
		{
			return $aDatas['value'];
		}
		return false;
	}
	
	function count()
	{
		return (count($this->aDatas));
	}
}


class NL_MySqlDataSource extends NL_DataSourceBase
{
	protected $iCount			= 0;
	protected $oResult			= null;
	protected $iNbRowsPerPage	= 0;
	protected $iIndex			= 0;
	
	public function NL_MySqlDataSource($sQuery = null)
	{
		parent::NL_DataSourceBase();
		$this->init($sQuery);
	}
	
	function init($sQuery)
	{
		$iPage					= (int) bab_rp('iPage', 1);
		$this->iNbRowsPerPage	= (int) bab_rp('iNbRowsPerPage', 10);
				
		//bab_debug($sQuery);
		
		global $babDB;
		$this->oResult = $babDB->db_query($sQuery);
		if(false !== $this->oResult)
		{
			$this->iCount = $babDB->db_num_rows($this->oResult);
			
			if(-1 !== $this->iNbRowsPerPage)
			{
				$sQuery .= ' LIMIT ' . (($iPage - 1) * $this->iNbRowsPerPage) . ', ' . $this->iNbRowsPerPage;
				
				//bab_debug($sQuery);
				$this->oResult = $babDB->db_query($sQuery);
			}
		}
	}
	
	public function count()
	{
		return $this->iCount;
	}
	
	
	public function getNbRowPerPage()
	{
		return $this->iNbRowsPerPage;
	}
	
	public function getNextItem()
	{
		if(false !== $this->oResult)
		{
			global $babDB;
			return $babDB->db_fetch_assoc($this->oResult);
		}
		return null;
	}
}

class NL_ColumnHeader
{
	private $sId = '';
	private $sText = '';
	private $sDataSourceFieldName = '';
	private $bUrl = false;
	private $bNoWrap = false;
	
	//Set javascript function for example to prevent 
	//the default behavior, the default behavior is in 
	//the function NL_MultiPageBase::getNextColumnHeader()
	private $sColumnHeaderUrl = '#'; 

	function NL_ColumnHeader($iId, $sText, $sDataSourceFieldName, $bUrl = false, $bNoWrap = false, $sLink = '#')
	{
		$this->setId($iId);
		$this->setText($sText);
		$this->setDataSourceFieldName($sDataSourceFieldName);
		$this->setUrlState($bUrl);
		$this->setLink($sLink); 
		$this->setNoWrap($bNoWrap);
	}

	function setId($sId)
	{
		$this->sId = $sId;
	}

	function getId()
	{
		return $this->sId;
	}

	function setText($sText)
	{
		$this->sText = (string) $sText;
	}

	function getText()
	{
		return $this->sText;
	}

	function setDataSourceFieldName($sDataSourceFieldName)
	{
		$this->sDataSourceFieldName = $sDataSourceFieldName;
	}

	function getDataSourceFieldName()
	{
		return $this->sDataSourceFieldName;
	}

	function setUrlState($bUrl)
	{
		$this->bUrl = (bool) $bUrl;
	}

	function getUrlState()
	{
		return (bool) $this->bUrl;
	}

	function setLink($sLink)
	{
		$this->sLink = (string) $sLink;
	}

	function getLink()
	{
		return (string) $this->sLink;
	}

	function setNoWrap($bNoWrap)
	{
		$this->bNoWrap = (bool) $bNoWrap;
	}

	function getNoWrap()
	{
		return (bool) $this->bNoWrap;
	}
}

class NL_MultiPageBase
{
	public $bIsPrevUrl = false;
	public $bIsNextUrl = false;
	public $bIsPageUrl = false;
	
	public $sPrevPageUrl = '';
	public $sNextPageUrl = '';
	public $sPageUrl = '';
	
	public $sPrevPageText = '';
	public $sNextPageText = '';
	public $sPageText = '';

	public $sTemplateFileName = 'multipage.html';
	public $sMultipageTemplate = 'multipage';
	public $sPaginationTemplate = 'pagination';
	public $sResultPerPage = '';
	public $sStatusLine = '';
	public $sPagination = '';
	
	public $iStartLink = 0;
	public $iEndLink = 0;
	public $iNbLink = 5;
	public $iNumOfPages = 0;
	
	public $iPage = 1;
	public $iNbRowsPerPage;
	public $iTotalNumOfRows = 0;
	
	public $aColumnHeaders = array();
	public $bIsColumnHeaderUrl = false;
	public $sColumnHeaderUrl = '';
	public $sColumnHeaderText = '';
	public $iColumnHeadersCount = 0;
	public $iColumnHeadersCountLessOne = 0;
	public $iColumnHeadersCountLessTwo = 0;
	
	public $bIsAltbg = true;
	public $sColumnData = '';
	
	public $oDataSource = null;
	var $aRow = null;
	
	public $aNbRowsPerPages;
	public $sNbRowPerPageSelected = '';
	
	public $sDisplay;
	
	public $sTg = '';
	public $sIdx = '';
	public $sOrderBy = '';
	public $sOrder = '';
	
	public $aActionsExcludeFilter = array();
	public $aActions = array();
	public $aFilteredActions = array();
	public $aActionItems = array();
	
	public $m_iDummy = 0;
	
	public $m_aAdditionnalPaginationAndFormParameters = array();
	public $m_sAdditionnalParameterName = '';
	public $m_sAdditionnalParameterValue = '';
	public $m_iAdditionnalParameterCount = 0;
	
	public $sSearchString = '';
	public $sSearchButtonCaption = '';

	public $oToolbar = null;

	public $bSearch		= true;
	public $sSearchData	= '';
	public $bNoWrap		= false;
	public $sResultMessage	= '&nbsp;';
	public $sSearchClass	= 'NL_searchNoResult';
	
	
	public $sRestoreBtnName	= '';
	public $sRestoreBtnAction	= '';
	public $sRestoreBtnCaption	= '';
	public $sRestoreBtnClass	= '';
	public $sRestoreBtnTitle	= '';
	
	public $bDeleteBtn			= false;
	public $sDeleteBtnName		= '';
	public $sDeleteBtnAction	= '';
	public $sDeleteBtnCaption	= '';
	public $sDeleteBtnClass	= '';
	public $sDeleteBtnTitle	= '';
	
	public $portlet = 0;
	
	function NL_MultiPageBase()
	{
		$this->sPrevPageText = nl_translate("Prcdent");
		$this->sNextPageText = nl_translate("Suivant");
		$this->sDisplay = nl_translate("Afficher");
		
		$this->aNbRowsPerPages = array(
			array('value' => 10, 'text' => '10'),
			array('value' => 20, 'text' => '20'),
			array('value' => 50, 'text' => '50'),
			array('value' => 100, 'text' => '100'),
			array('value' => 200, 'text' => '200'),
			array('value' => 300, 'text' => '300'),
			array('value' => 400, 'text' => '400'),
			array('value' => 500, 'text' => '500'),
			array('value' => -1, 'text' => nl_translate("Tous")));
			
		$this->sResultPerPage = nl_translate("Rsultat(s) par page");
		
		$this->setColumnDataSource(new NL_DataSourceBase());
		if(isset($_GET['_onNbRowPerPageChanged']))
		{
			$this->iPage = 1;
		}
		else
		{
			$this->iPage = (int) bab_rp('iPage', 1);	
		}
		
		$this->iNbRowsPerPage = (int) bab_rp('iNbRowsPerPage', 10);
		
		$this->sTg		= bab_rp('tg', '');
		$this->sIdx		= bab_rp('idx', '');
		$this->sOrderBy	= (string) bab_rp('sOrderBy', '');
		$this->sOrder	= (string) bab_rp('sOrder', '');
		
		$this->m_sAdditionnalParameterName = '';
		$this->m_sAdditionnalParameterValue = '';

		$this->sSearchString = bab_rp('sSearch', '');
		$this->sSearchButtonCaption = nl_translate("Trouver");
		
		$this->sDeleteBtnName		= 'idx';
		$this->sDeleteBtnCaption	= nl_translate("Suppr.");
		$this->sDeleteBtnAction 	= '';

		$this->sRestoreBtnName 		= 'idx';
		$this->sRestoreBtnCaption 	= nl_translate("Rest.");
		$this->sRestoreBtnAction 	= '';
	}

	function getNextColumnHeader()
	{
		$this->bIsColumnHeaderUrl = false;

		$aDatas = each($this->aColumnHeaders);
		
		if(false !== $aDatas)
		{
			$oColumnHeader =& $aDatas['value'];
			
			if(true === $oColumnHeader->getUrlState())
			{
				$this->bIsColumnHeaderUrl = true;
				
				if('#' === $oColumnHeader->getLink())
				{
					$this->sColumnHeaderUrl = $this->buildPageUrl($this->iPage, false);
					$this->sColumnHeaderUrl = ereg_replace('&sOrderBy=[^&.]+', '', $this->sColumnHeaderUrl);
					$this->sColumnHeaderUrl = ereg_replace('&sOrder=[^&.]+', '', $this->sColumnHeaderUrl);
					$this->sColumnHeaderUrl .= '&sOrderBy=' . $oColumnHeader->getDataSourceFieldName();

					if($this->sOrderBy === (string) $oColumnHeader->getDataSourceFieldName())
					{
						if($this->sOrder === (string) 'ASC')
						{
							$this->sColumnHeaderUrl .= '&sOrder=' . 'DESC';
						}
						else 
						{
							$this->sColumnHeaderUrl .= '&sOrder=' . 'ASC';
						}
					}
					else 
					{
						$this->sColumnHeaderUrl .= '&sOrder=' . 'ASC';
					}
					$this->sColumnHeaderUrl = htmlentities($this->sColumnHeaderUrl);
				}
				else
				{
					$this->sColumnHeaderUrl = htmlentities($oColumnHeader->getLink());
				}
			}
			else
			{
				$this->sColumnHeaderUrl = '#';
			}
			
			$this->sColumnHeaderText = $oColumnHeader->getText();
			return true;
		}
		
		reset($this->aColumnHeaders);
		return false;
	}
	
	function getNextRow()
	{
		//bab_debug(__FUNCTION__);
		if($this->oDataSource instanceof NL_DataSourceBase)
		{
			$this->aRow = $this->oDataSource->getNextItem();
			//bab_debug($this->aRow);
			
			if(false !== $this->aRow && null !== $this->aRow)
			{
				$this->bIsAltbg = !$this->bIsAltbg;
				return true;
			}
		}
		return false;
	}
	
	function getNextColumnData()
	{
		$this->bNoWrap = false;
		
		$aHeaders = each($this->aColumnHeaders);
		if(false !== $aHeaders)
		{
			$oColumnHeader =& $aHeaders['value'];
			$this->sColumnData = '???';
			if(isset($this->aRow[$oColumnHeader->getDataSourceFieldName()]))
			{
				$this->sColumnData = $this->aRow[$oColumnHeader->getDataSourceFieldName()];
				$this->bNoWrap = $oColumnHeader->getNoWrap();
				
				///bab_debug('bNowrap ==> ' . (($this->bNoWrap) ? 'Yes' : 'No') . ' ' . $oColumnHeader->getDataSourceFieldName());
			}
			return true;
		}
		reset($this->aColumnHeaders);
		return false;
	}
		
	function getNextNbRow()
	{
		$aDatas = each($this->aNbRowsPerPages);
		if(false !== $aDatas)
		{
			$this->iNbRow = $aDatas['value']['value'];
			$this->sNbRow = $aDatas['value']['text'];
			
			$this->sNbRowPerPageSelected = ($this->iNbRowsPerPage == $this->iNbRow) ? 'selected="selected"' : '';
			
			return true;
		}
		return false;
	}

	function addColumnHeader($oColumnHeader)
	{
		if($oColumnHeader instanceof NL_ColumnHeader)
		{
			$this->aColumnHeaders[$oColumnHeader->getDataSourceFieldName()] = $oColumnHeader;
		}
	}
	
	function setColumnDataSource($oDataSource)
	{
		if($oDataSource instanceof NL_DataSourceBase)
		{
			$this->oDataSource = $oDataSource;
			$this->iTotalNumOfRows = $oDataSource->count();
		}
	}
	
	function processActionsExcludeFilter()
	{
		$aActions = $this->aActions;
		
		foreach($this->aActionsExcludeFilter as $key => $value)
		{
			if(isset($aActions[$value['sActionId']]))
//			if(array_key_exists($value['sActionId'], $aActions))
			{
			if(isset($this->aRow[$value['sDataSourceFieldName']]))
//			if(array_key_exists($value['sDataSourceFieldName'], $this->aRow))
			{
					$sCondition = 'return ' . $this->aRow[$value['sDataSourceFieldName']] . ' ' . $value['sOperator'] . ' ' . $value['sDataSourceFieldValue'] . ';';
					
					$sCondition = eval($sCondition);
					if($sCondition && !bab_isAccessValid(NL_INSCRIPTION_GROUP, $this->aRow['iId']) )
					{
						unset($aActions[$value['sActionId']]);
					}
				}
			}
		}
		$this->aFilteredActions = $aActions;
		return false; //for the template
	}
	
	function addActionExcludeFilter($sActionId, $sDataSourceFieldName, $sOperator, $sDataSourceFieldValue)
	{
		$this->aActionsExcludeFilter[] = array('sActionId' => $sActionId, 'sDataSourceFieldName' => $sDataSourceFieldName, 
			'sOperator' => $sOperator, 'sDataSourceFieldValue' => $sDataSourceFieldValue);
	}
	
	function addAction($sId, $sText, $sIcon, $sLink, $aDataSourceFields)
	{
		$this->aActions[$sId] = array('sId' => $sId, 'sText' => $sText, 'sIcon' => $sIcon, 'sLink' => $sLink, 'aDataSourceFields' => $aDataSourceFields);
	}

	function getNextAction(&$bSkip)
	{
		$datas = each($this->aFilteredActions);
		if(false != $datas)
		{
			$this->aActionItems = $datas['value'];
			$aDataSourceFields = $this->aActionItems['aDataSourceFields'];
			
			foreach($aDataSourceFields as $key => $value)
			{
				if(isset($this->aRow[$value['sDataSourceFieldName']]))
				{
					$this->aActionItems['sLink'] .= '&' . $value['sUrlParamName'] . '=' . $this->aRow[$value['sDataSourceFieldName']];
				}
			}
			return true;				
		}
		reset($this->aFilteredActions);
		reset($this->aActions);
		return false;
	}
	
	function getNextAdditionnalParameter()
	{
		$datas = each($this->m_aAdditionnalPaginationAndFormParameters);
		if(false != $datas)
		{
			$this->m_sAdditionnalParameterName = htmlentities($datas['key']);
			$this->m_sAdditionnalParameterValue = htmlentities($datas['value']);
			return true;
		}
		else
		{
			$this->m_sAdditionnalParameterName = '';
			$this->m_sAdditionnalParameterValue = '';
			reset($this->m_aAdditionnalPaginationAndFormParameters);
			return false;
		}
	}

	/* calcul l'offset de dbut et de fin */
	function computeStartEndPos()
	{
		$this->iNumOfPages	= 0;
		$this->iEndLink		= 0;
		$this->iStartLink	= 1;

		if( $this->iNbRowsPerPage > 0 && $this->iTotalNumOfRows > $this->iNbRowsPerPage )
		{
			$this->iNumOfPages = ceil($this->iTotalNumOfRows / $this->iNbRowsPerPage);
		}
		else if($this->iTotalNumOfRows == $this->iNbRowsPerPage)
		{
			$this->iPage = $this->iNumOfPages = 1;
		}

		if($this->iPage >= $this->iNbLink)
		{
			$this->iStartLink = $this->iPage - $this->iNbLink;
			if(0 >= $this->iStartLink)
			{
				$this->iStartLink = 1;
			}
		}

		if(($this->iPage + $this->iNbLink) > $this->iNumOfPages)
		{
			$this->iEndLink = $this->iNumOfPages;
		}
		else
		{
			$this->iEndLink = $this->iPage + $this->iNbLink;
		}
	}	

	function buildPrevPageUrl()
	{
		if( $this->iPage > 1 && $this->iPage <= $this->iNumOfPages)
		{
			$iPrevPage = $this->iPage - 1;
			$this->bIsPrevUrl = true;
			$this->sPrevPageUrl	= $this->buildPageUrl($iPrevPage);
			$this->sPrevPageText = nl_translate("Prcdent");
		}
		else
		{	
			$this->bIsPrevUrl = false;
			$this->iPage = 1;
			$this->sPrevPageText = '';
		}
	}

	function buildNextPageUrl()
	{
		if( $this->iPage >= 1 && $this->iPage < $this->iNumOfPages && 
			(($this->iTotalNumOfRows - ($this->iNbRowsPerPage * $this->iPage)) > 0) 
		  )
		{
			$iNextPage = $this->iPage + 1;
			$this->bIsNextUrl = true;
			$this->sNextPageUrl	= $this->buildPageUrl($iNextPage);
			$this->sNextPageText = nl_translate("Suivant");
		}
		else
		{
			$this->bIsNextUrl = false;
			$this->sNextPageText = '';
		}
	}
	
	function getNextPage()
	{
		$this->sPageText = $this->iStartLink;

		if($this->iStartLink != $this->iPage)
		{
			$this->bIsPageUrl = true;
			$this->sPageUrl = $this->buildPageUrl($this->iStartLink);
		}
		else
		{
			$this->bIsPageUrl = false;
		}

		if($this->iStartLink <= $this->iEndLink)
		{
			$this->iStartLink++;
			return true;
		}
		else 
		{
			return false;
		}
	}

	/**
	 * Creates an url by concatenating the baseUrl and and the parameters.
	 * The parameters are in an array where the key will be used as the name of the parameter
	 * and the value as the parameters value.
	 * The function checks if the baseUrl already contains parameters in which case the parameters
	 * are appended after a '&'. If baseUrl does not contain any parameter, the parameters are
	 * appended after a '?'.
	 * @param string $baseUrl
	 * @param array $parameters
	 * @return string
	 */
	function createUrl($baseUrl, $parameters)
	{
		$l = array();
		$url = $baseUrl;
		if ($parameters)
		{
			foreach ($parameters as $paramaterName => $paramaterValue)
				$l[] = $paramaterName . '=' . $paramaterValue;
			if (strpos($url, '?') === false)
				$url .= '?';
			else
				$url .= '&';
			$url .= implode('&', $l);
		}
		return $url;	
	}			

	function buildPageUrl($iPageNumber, $bUseHtmlEntities = true)
	{
		$sPageUrl = ereg_replace('\?tg=[^&.]+', '', $_SERVER['REQUEST_URI']);
		$sPageUrl = ereg_replace('&iPage=[^&.]+', '', $sPageUrl);
		$sPageUrl = ereg_replace('&iNbRowsPerPage=[^&.]+', '', $sPageUrl);
		$sPageUrl = ereg_replace('&sSearch=[^&.]+', '', $sPageUrl);
		$sPageUrl = ereg_replace('&sOrderBy=[^&.]+', '', $sPageUrl);
		$sPageUrl = ereg_replace('&sOrder=[^&.]+', '', $sPageUrl);
		
		$sExtraParams = '';
		reset($this->m_aAdditionnalPaginationAndFormParameters);
		foreach($this->m_aAdditionnalPaginationAndFormParameters as $sName => $sValue)
		{
			$sPageUrl = ereg_replace('&' . $sName . '=[^&.]+', '', $sPageUrl);
			$sExtraParams .= '&' . $sName . '=' . urlencode($sValue);
		}
		
		$sPageUrl = '?tg=' . urlencode($this->sTg);
		
		if(true === $bUseHtmlEntities)
		{
			return htmlentities($sPageUrl .= '&iPage=' . urlencode($iPageNumber) . '&iNbRowsPerPage=' . 
				urlencode($this->iNbRowsPerPage) . '&sSearch=' . urlencode($this->sSearchString) . 
				'&sOrderBy=' . urlencode($this->sOrderBy) . '&sOrder=' . urlencode($this->sOrder) . 
				$sExtraParams);
		}
		else 
		{
			return $sPageUrl .= '&iPage=' . urlencode($iPageNumber) . '&iNbRowsPerPage=' . 
				urlencode($this->iNbRowsPerPage) . '&sSearch=' . urlencode($this->sSearchString) . 
				'&sOrderBy=' . urlencode($this->sOrderBy) . '&sOrder=' . urlencode($this->sOrder) . 
				$sExtraParams;
		}
	}
	
	function getPagination()
	{
		$this->buildPrevPageUrl();
		$this->buildNextPageUrl();
		return bab_printTemplate($this, nl_relativePath() . $this->sTemplateFileName, $this->sPaginationTemplate);
	}

	function printTemplate()
	{
		$this->iColumnHeadersCount = count($this->aColumnHeaders);
		$this->iColumnHeadersCountLessOne = $this->iColumnHeadersCount - 1;
		$this->iColumnHeadersCountLessTwo = $this->iColumnHeadersCount - 2;
		
		$this->computeStartEndPos();
		$this->sStatusLine = $this->getStatusLine();
		$this->sPagination = $this->getPagination();
		
		reset($this->m_aAdditionnalPaginationAndFormParameters);
		return bab_printTemplate($this, nl_relativePath() . $this->sTemplateFileName, $this->sMultipageTemplate);
	}
		
	function getStatusLine()
	{
		if($this->iNbRowsPerPage > 0)
		{
			$iNumOfPages = ceil($this->iTotalNumOfRows / $this->iNbRowsPerPage);
			$iStart = (($this->iPage * $this->iNbRowsPerPage) - $this->iNbRowsPerPage) + 1;
			$iEnd   = ($this->iPage * $this->iNbRowsPerPage);
		}

		$sResults = nl_translate("Rsultats");
		$sResult = nl_translate("Resultat");
		$sPages = nl_translate("Pages");
		$sPage = nl_translate("Page");
		
		//-1 == ALL
		if(-1 != $this->iNbRowsPerPage)
		{
			$sString = nl_translate("Rsultat(s) de %d  %d (%d %s / %d %s )");
			return htmlentities(sprintf($sString, $iStart, $iEnd, $this->iTotalNumOfRows, 
				(((1 < $this->iTotalNumOfRows) ? $sResults : $sResult)), $iNumOfPages, 
				((1 < $iNumOfPages) ? $sPages : $sPage)));
		}
		else
		{
			$sString = nl_translate("Tous les rsultats (%d %s / 1 %s)");
			return htmlentities(sprintf($sString, $this->iTotalNumOfRows, 
				((1 < $this->iTotalNumOfRows) ? $sResults : $sResult), $sPage));
		}
	}
	
	function addPaginationAndFormParameters($sName, $sValue)
	{
		$this->m_aAdditionnalPaginationAndFormParameters[$sName] = $sValue;
	}

	function setNoSearchResultMessage($sMessage)
	{
		$this->sResultMessage = $sMessage;
		$this->sSearchClass	= 'NL_searchResult';
	}
}
