<?php
//-------------------------------------------------------------------------
// OVIDENTIA http://www.ovidentia.org
// Ovidentia is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
// USA.
//-------------------------------------------------------------------------
/**
 * @license http://opensource.org/licenses/gpl-license.php GNU General Public License (GPL)
 * @copyright Copyright (c) 2012 by CANTICO ({@link http://www.cantico.fr})
 */
require_once 'base.php';
global $babInstallPath;
require_once $babInstallPath.'admin/acl.php';
require_once dirname(__FILE__) . '/nlincl.php';

bab_Functionality::includefile('PortletBackend');

/**
 * 
 */
class Func_PortletBackend_Newsletter extends Func_PortletBackend
{
	public function getDescription()
	{
		return nl_translate('Newsletter portlet');
	}
	
	public function select($category = null)
	{
		return array(
			'newsletter' => $this->portlet_Newsletter(
				'newsletter',
				nl_translate('Newsletter portlet'),
				nl_translate('A portlet containing the list of newsletter which users can subscribs')
			)
		);
	}
	
	
	
	/**
	 * Get portlet definition instance
	 * @param	string	$portletId			portlet definition ID
	 *
	 * @return portlet_PortletDefinitionInterface
	 */
	public function getPortletDefinition($portletId)
	{
		return $this->portlet_Newsletter(
			'newsletter',
			nl_translate('Newsletter portlet'),
			nl_translate('A portlet containing the list of newsletter which users can subscribs')
		);
	}


	public function portlet_Newsletter($id, $name, $description)
	{
		return new PortletDefinition_Newsletter($id, $name, $description);
	}
	
	public function getConfigurationActions()
	{
		return array();
	}
}


/////////////////////////////////////////


class PortletDefinition_Newsletter implements portlet_PortletDefinitionInterface
{
	
	private $id;
	private $name;
	private $description;
	
	public function __construct($id, $name, $description)
	{
		$this->id = $id;
		$this->name = $name;
		$this->description = $description;
	}

	public function getId()
	{
		return $this->id; 
	}
	
	
	public function getName()
	{
		return $this->name;
	}


	public function getDescription()
	{
		return $this->description;	
	}


	public function getPortlet()
	{		
		return new Portlet_Newsletter($this->id, $this->name, $this->description);
	}
	
	public function getPreferenceFields()
	{
		global $babDB;
		
		$options = array();
		
		$sql = "SELECT * FROM nl_newsletter_topic";
		
		$res = $babDB->db_query($sql);
		
		while($arr = $babDB->db_fetch_array($res)){
			$options[] = array('value' => $arr['id'], 'label' => $arr['name']);
		}
		$return = array(
			array(
				'label' => 'Lettres (si aucune lettre n\'est slctionn toutes les lettres seront retournes)',
				'type' => 'multiselect',
				'name' => 'ids',
				'options' => $options
			)
		);
		
		return $return;
	}
	
	/**
	 * Returns the widget rich icon URL.
	 * 128x128 ?
	 * 
	 * @return string
	 */
	public function getRichIcon()
	{
		return $GLOBALS['babSkinPath'] . 'images/addons/newsletter/mail-icon.png';
	}
	
	
	/**
	 * Returns the widget icon URL.
	 * 16x16 ?
	 *
	 * @return string
	 */
	public function getIcon()
	{
		return $GLOBALS['babSkinPath'] . 'images/addons/newsletter/mail-icon.png';
	}
	
	/**
	 * Get thumbnail URL
	 * max 120x60
	 */
	public function getThumbnail()
	{
		return '';
	}
	
	public function getConfigurationActions()
	{
		return array();
	}
}




class Portlet_Newsletter extends Widget_Item implements portlet_PortletInterface
{
	private $id;
	private $name;
	private $description;
	private $options;
	
	/**
	 * Instanciates the widget factory.
	 *
	 * @return Func_Widgets
	 */
	function Widgets()
	{
		return bab_Functionality::get('Widgets');
	}
	
	/**
	 */
	public function __construct($id, $name, $description)
	{
		$this->id = $id;
		$this->name = $name;
		$this->description = $description;
	}

	/**
	 * @param Widget_Canvas	$canvas
	 * @ignore
	 */
	public function display(Widget_Canvas $canvas)
	{
		$W = $this->Widgets();
		
		require_once dirname(__FILE__) . '/helper/userNewsletterListDataSource.class.php';
		
		$ids = null;
		if(isset($this->options['ids']) && !empty($this->options['ids'])){
			$ids = $this->options['ids'];
		}
		
		$oMultiPage = new NL_MultiPageBase();
		
		$oMultiPage->iNbRowsPerPage = -1;
		$oMultiPage->portlet = 1;
		$oMultiPage->sIdx = 'displayNewsletterList';
		
		$oMultiPage->setColumnDataSource(new NL_UserNlListDataSource($ids));
		$oMultiPage->addColumnHeader(new NL_ColumnHeader(0, nl_translate("Name"), 'sName', false));
		$oMultiPage->addColumnHeader(new NL_ColumnHeader(1, nl_translate("Description"), 'sDescription', false));
		
		//To bypass a bug in the NL_MultiPageBase class, the idx is not passed
		//in the prev and next link
		$oMultiPage->addPaginationAndFormParameters('idx', 'displayNewsletterList');
		
		$sTg = 'addon/newsletter/users';
		
		//*
		//Pour les icnes
		{
			$aDataSourceFields = array(
					array('sDataSourceFieldName' => 'iId', 'sUrlParamName' => 'id')
			);
		
			if(true === bab_userIsloggedin())
			{
				$oMultiPage->addActionExcludeFilter('suscribe', 'iIdSus', '!=', '0');
				$oMultiPage->addActionExcludeFilter('unsuscribe', 'iIdSus', '==', '0');
			}
		
			$sLink = $GLOBALS['babUrlScript'] . '?tg=' . urlencode($sTg) . '&idx=' . urlencode('subscribe');
			$oMultiPage->addAction('suscribe', nl_translate("Subscribe"),
					$GLOBALS['babSkinPath'] . 'images/addons/newsletter/mail_get.png',
					$sLink, $aDataSourceFields);
				
			$sLink = $GLOBALS['babUrlScript'] . '?tg=' . urlencode($sTg) . '&idx=' . urlencode('unsubscribe');
			$oMultiPage->addAction('unsuscribe', nl_translate("unsubscribe"),
					$GLOBALS['babSkinPath'] . 'images/addons/newsletter/mail_delete.png',
					$sLink, $aDataSourceFields);
		}
		
		$this->item = $W->Html($oMultiPage->printTemplate());
		
		return $this->item->display($canvas);
	}

	public function getName()
	{
		return get_class($this);
	}

	public function getPortletDefinition()
	{
		return new PortletDefinition_Newsletter($this->id, $this->name, $this->description);
	}
	
	/**
	 * receive current user configuration from portlet API
	 */
	public function setPreferences(array $configuration)
	{
		$this->options = $configuration;
	}
	
	public function setPreference($name, $value)
	{
		$this->options[$name] = $value;
	}
	
	public function setPortletId($id)
	{
		
	}
}